# coding=utf-8
import json
import logging

__author__ = 'ThucNC'

import configparser
from typing import List, Union

import requests
from ptoolbox.dsa.dsa_problem import DsaProblem
from ptoolbox.helpers.clog import CLog
from ptoolbox.models.general_models import Problem, TestCase

_logger = logging.getLogger(__name__)


class UCode:
    def __init__(self, base_url, token):
        self.s = requests.session()
        self.api_base_url = base_url
        self.token = token
        self._headers = {
            'access-token': self.token
        }

    def get_api_url(self, path):
        return self.api_base_url + path

    def create_problem(self, lesson_id, problem: Union[Problem, str], score=10, xp=100):
        if isinstance(problem, str):
            problem: Problem = DsaProblem.load(problem, load_testcase=True)

        data = {
            "name": problem.name,
            "type": "code",
            "statement": problem.statement,
            "statement_format": "markdown",
            "input_desc": problem.input_format,
            "output_desc": problem.output_format,
            "constraints": problem.constraints,
            "compiler": "python",
            "statement_language": "vi",
            "score": score,
            "solution": problem.solution,
            "status": "string",
            "visibility": "public",
            "ucoin": xp
        }

        url = self.get_api_url(f"/lesson-item/{lesson_id}/question")
        response = self.s.post(url, json=data, headers=self._headers)

        print(response.status_code)
        res = response.json()
        print(res)
        if res['success']:
            question_id = res['data']['id']
            print("question_id:", question_id)
        else:
            raise Exception("Cannot create question:" + json.dumps(res))

        # upload testcase
        for testcase in problem.testcases:
            self.upload_testcase(question_id, testcase)

        return question_id

    def create_problems(self, lesson_id, problems: List[Problem], score=10, xp=100):
        question_ids = []
        for problem in problems:
            q_id = self.create_problem(self, lesson_id, problem, score, xp)
            question_ids.append(q_id)

        return question_ids

    def upload_testcase(self, problem_id, testcase: TestCase, is_sample=True, score=10):
        url = self.get_api_url(f"/question/{problem_id}/testcase")
        data = {
            "name": testcase.name,
            "explanation": testcase.explanation,
            "input": testcase.input,
            "output": testcase.output,
            "score": score,
            "is_sample": bool(is_sample)
        }

        response = self.s.post(url, json=data, headers=self._headers)
        print(response.status_code)
        res = response.json()
        print(res)
        if res['success']:
            testcase_id = res['data']['id']
            print("testcase_id:", testcase_id)
        else:
            CLog.error("Cannot create testcase:" + json.dumps(res))

    @staticmethod
    def read_credential(credential_file):
        config = configparser.ConfigParser()
        config.read(credential_file)
        if not config.has_section('UCODE'):
            CLog.error(f'Section `UCODE` should exist in {credential_file} file')
            return None, None
        if not config.has_option('UCODE', 'api_url') or not config.has_option('UCODE', 'token'):
            CLog.error(f'api_url and/or token are missing in {credential_file} file')
            return None, None

        api_url = config.get('UCODE', 'api_url')
        token = config.get('UCODE', 'token')

        return api_url, token


if __name__ == "__main__":
    ucode = UCode("https://dev-api.ucode.vn/api", "72821b59462c5fdb552a049c1caed85c")
    # problem_folder = "../../problems/domino_for_young"
    problem_folder = "/home/thuc/projects/ucode/courses/course-py101/lesson2/c1_input/p13_chao_ban"
    ucode_lesson_id = 172

    problem: Problem = DsaProblem.load(problem_folder, load_testcase=True)
    #
    # print(problem)
    #
    print(len(problem.testcases))
    for testcase in problem.testcases:
        print(testcase)


    # ucode.create_problem(lesson_id=172, problem=problem_folder)




# coding=utf-8
import logging
import os
from distutils.dir_util import copy_tree

from ptoolbox.codeforces.codeforces import Codeforces
from ptoolbox.dsa.dsa_problem import DsaProblem

__author__ = 'ThucNC'

from ptoolbox.kattis.kattis import Kattis

_logger = logging.getLogger(__name__)


def read_all_problems(base_folder, nested_folder=0, load_testcase=False):
    res = []
    problem_folders = [f.path for f in os.scandir(base_folder) if f.is_dir()]
    for problem_folder in sorted(problem_folders):
        if nested_folder:
            for i in range(nested_folder):
                subfolders = [f.path for f in os.scandir(problem_folder) if f.is_dir()]
                for folder in sorted(subfolders):
                    print(problem_folder)
                    problem = DsaProblem.load(os.path.join(problem_folder, folder), load_testcase=load_testcase)
                    res.append((os.path.join(problem_folder, folder), problem))
        else:
            print(problem_folder)
            problem = DsaProblem.load(problem_folder, load_testcase=load_testcase)
            res.append((problem_folder, problem))
    return res


def get_all_codeforce_problem(base_folder):
    problem_folders = [f.path for f in os.scandir(base_folder) if f.is_dir()]
    cf = Codeforces()
    for problem_folder in sorted(problem_folders):
        problem = DsaProblem.load(problem_folder)
        if problem.src_url:
            output_folder = os.path.join(os.path.dirname(problem_folder), os.path.basename(problem_folder).lower())
            print("Getting", problem.src_url, "from", problem_folder, "to", output_folder)
            dsa_problem = cf.get_problem_from_url(problem.src_url)
            DsaProblem.save(dsa_problem, output_folder, None, overwrite=False)


def get_all_kattis_problem(base_folder):
    problem_folders = [f.path for f in os.scandir(base_folder) if f.is_dir()]
    kt = Kattis()
    for problem_folder in sorted(problem_folders):
        problem = DsaProblem.load(problem_folder)
        if problem.src_url:
            dsa_problem = kt.get_problem(problem.src_url)
            difficulty = int(Kattis.original_difficulty(dsa_problem.difficulty) * 10)
            output_folder = os.path.join(os.path.dirname(base_folder), "new",
                                         f"difficulty_{difficulty}")
            print("Getting", problem.src_url, "from", problem_folder, "to", output_folder)
            problem_code = DsaProblem.make_problem_code(dsa_problem.name)
            DsaProblem.save(dsa_problem, output_folder, problem_code, overwrite=True)
            copy_tree(problem_folder, os.path.join(output_folder, problem_code, "_vi"))
        # break


def rearrange_codeforce_problems():
    problems = read_all_problems("/home/thuc/projects/ucode/dsa-problems/codeforces_new", 1)

    for prob in problems:
        difficulty = Codeforces.original_difficulty(prob[1].difficulty)
        problem_folder = os.path.dirname(prob[0])
        problem_name = os.path.basename(prob[0])
        print("current path:", problem_folder, "problem_name:", problem_name)
        old_problem_folder = f"/home/thuc/projects/ucode/dsa-problems/codeforces/" \
                             f"{os.path.basename(problem_folder).capitalize()}"
        print("old path:", old_problem_folder)
        if not os.path.exists(old_problem_folder):
            raise Exception(f"not found: {old_problem_folder}")
        new_base = f"/home/thuc/projects/ucode/dsa-problems/codeforces/difficulty_{difficulty}"

        copy_tree(os.path.join(problem_folder, problem_name), os.path.join(new_base, problem_name))
        copy_tree(old_problem_folder, os.path.join(new_base, problem_name, "_vi"))
        print("new base:", new_base)
        if not os.path.exists(new_base):
            os.makedirs(new_base)
        print(prob[1].src_url, '-', difficulty, '-', prob[0])


if __name__ == "__main__":
    # list all problem
    # folder = "/home/thuc/projects/ucode/dsa-problems/codeforces"
    # problems = read_all_problems(folder, 1)
    # for prob in problems:
    #     difficulty = Codeforces.original_difficulty(prob[1].difficulty)
    #     print(prob[1].src_url, '-', difficulty, '-', prob[1].name, '-', ",".join(prob[1].tags), '-', prob[0])
    # rearrange_codeforce_problems()
    # get_all_codeforce_problem(folder)

    get_all_kattis_problem("/home/thuc/projects/ucode/dsa-problems/katis/unsorted")



import copy
import html

import requests
import tomd
from bs4 import BeautifulSoup

from ptoolbox.dsa.dsa_problem import DsaProblem
from ptoolbox.helpers.clog import CLog
from ptoolbox.models.general_models import Problem, TestCase


def to_markdown(html_text):
    if not html_text:
        return ""

    html_text = html_text.replace("$$$", "$")
    soup = BeautifulSoup(html_text, 'html.parser')

    # img tag must be inside <p> tag
    imgs = soup.select("img")
    if imgs:
        for img in imgs:
            if "alt" in img:
                img.string = img['alt']
            del img['style']
            del img['alt']
            new_tag = soup.new_tag('p')
            img.wrap(new_tag)

    bold = soup.select('.tex-font-style-bf')
    for tag in bold:
        tag.name = 'strong'
        tag.attrs = {}
    tt = soup.select('.tex-font-style-tt')
    for tag in tt:
        tag.name = 'code'
        tag.attrs = {}
    tt = soup.select('.tex-font-style-it')
    for tag in tt:
        tag.name = 'i'
        tag.attrs = {}

    # convert subscript and superscript to latext
    #  *...*<sub class="lower-index">*...*
    #  *...*<sub class="upper-index">*...*
    subscripts = soup.select('sub')
    for subscript in subscripts:
        print(str(subscript))
    supper_scripts = soup.select('sup')
    for supper_script in supper_scripts:
        print(str(supper_script))

    res = tomd.convert(html.unescape(soup.decode())).strip()


    return res


class Codeforces:
    def __init__(self):
        self.s = requests.session()
        self._headers = {
            'user-agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.77 Safari/537.36',
        }
        self.csrf = ''

    def get_problem_list(self, page_index):
        url = f'https://codeforces.com/problemset/page/{page_index}'

        headers = copy.deepcopy(self._headers)
        r = self.s.get(url, headers=headers)

        raw = r.text
        # print(raw)

        soup = BeautifulSoup(raw, 'html.parser')
        self.csrf = soup.select('span.csrf-token')[0]['data-csrf']
        # print('CSRF:', self.csrf)

        pagination = soup.select('div.pagination > ul > li')
        last_page = pagination[-2]
        # print(last_page)
        last_page_index = int(last_page.select('span.page-index')[0]['pageindex'])
        if page_index>last_page_index:
            CLog.warn(f'Overpass last page: #{page_index}/{last_page_index}')
            return [], last_page_index
        CLog.info(f'Getting problem list from page: #{page_index}/{last_page_index}')

        problem_tags = soup.select('table.problems > tr')[1:] # remove header line
        # print(len(problem_tags))
        # print(problem_tags[0])

        problems = []
        for problem_tag in problem_tags:
            problem = Problem()
            pid = problem_tag.select('td.id a')[0]

            problem.src_id = pid.text.strip()
            problem.src_url = 'https://codeforces.com' + pid['href']

            ptitle = problem_tag.select('td')[1].select('div')[0]
            problem.name = ptitle.select('a')[0].text.strip()

            ptags = problem_tag.select('td')[1].select('div')[1].select('a')
            for ptag in ptags:
                tag = ptag.text.strip()
                problem.tags.append(tag)

            difficulty = problem_tag.select('td')[3].select('span')
            if difficulty:
                difficulty = difficulty[0].text.strip()
                difficulty = int(difficulty) ** 0.5
                dmax = 3800 ** 0.5
                dmin = 250 ** 0.5

                difficulty = (difficulty - dmin)/(dmax-dmin) * 10
            else:
                difficulty = 0

            problem.difficulty = difficulty

            submission_url = problem_tag.select('td')[4].select('a')
            if submission_url:
                problem.src_status_url = 'https://codeforces.com' + submission_url[0]['href']

            problems.append(problem)

            # print(problem.name)
            # print(problem.src_id)
            # print(problem.src_url)
            # print(problem.tags)
            # print(problem.difficulty)
            # break
        return problems, last_page_index

    def get_problem_from_url(self, url):
        problem = Problem()
        problem.src_url, problem.src_status_url = Codeforces.parse(url)
        # print(problem.src_url)
        CLog.info("Getting problem from url: " + problem.src_url)
        url = problem.src_url
        headers = copy.deepcopy(self._headers)
        r = self.s.get(url, headers=headers)
        raw = r.text
        soup = BeautifulSoup(raw, 'html.parser')
        problem_statement = soup.select('div.problem-statement')[0]
        print(problem_statement)

        title = problem_statement.select('div.title')[0]
        title.extract()
        title = title.text
        if title[1] == "." and 'A' <= title[0] <= 'Z':
            title = title[2:].strip()
        problem.src_name = problem.name = title

        limit_time = problem_statement.select('div.time-limit')[0]
        limit_time.select('div')[0].extract()
        problem.limit_time = int(''.join(c for c in limit_time.text if c in '0123456789')) * 1000

        limit_memory = problem_statement.select('div.memory-limit')[0]
        limit_memory.select('div')[0].extract()
        # problem.limit_memory = limit_memory.text
        problem.limit_memory = int(''.join(c for c in limit_memory.text if c in '0123456789'))

        input_type = problem_statement.select('div.input-file')[0]
        input_type.select('div')[0].extract()
        problem.input_type = input_type.text

        output_type = problem_statement.select('div.output-file')[0]
        output_type.select('div')[0].extract()
        problem.output_type = output_type.text

        if problem.input_type.strip() == 'standard input':
            problem.input_type = 'stdin'
        if problem.output_type.strip() == 'standard output':
            problem.output_type = 'stdout'

        problem_statement.select('div.header')[0].extract()

        input_format = problem_statement.select('div.input-specification')[0]
        input_format.select('div.section-title')[0].extract()
        problem.input_format = input_format.decode_contents()
        input_format.extract()

        output_format = problem_statement.select('div.output-specification')[0]
        output_format.select('div.section-title')[0].extract()
        problem.output_format = output_format.decode_contents()
        output_format.extract()

        sample_tests = problem_statement.select('div.sample-tests')[0]
        inputs = [t.decode_contents().replace("<br/>", "\n").strip() for t in sample_tests.select('div.input > pre')]
        outputs = [t.decode_contents().replace("<br/>", "\n").strip() for t in sample_tests.select('div.output > pre')]
        sample_tests.extract()

        for i in range(len(inputs)):
            # print("input: ", inputs[i])
            # print("output: ", outputs[i])
            testcase = TestCase(inputs[i], outputs[i], name=str(i+1))
            problem.stock_testcases.append(testcase)

        note = problem_statement.select('div.note')
        if note:
            note = note[0]
            note.select('div.section-title')[0].extract()
            # print(note)
            explanations = note.select('p')
            for i in range(len(inputs)):
                if i<len(explanations):
                    problem.stock_testcases[i].explanation = to_markdown(explanations[i].decode())
                    # print(problem.stock_testcases[i].explanation)

            note.extract()
            problem.testcases_sample_note = note.decode_contents()

        problem.statement = problem_statement.select('div:not([class])')[0].decode_contents()

        problem.difficulty = 0
        ptags = soup.select('div.sidebox div.roundbox span.tag-box')
        for ptag in ptags:
            tag = ptag.text.strip()
            if tag[0]=='*' and tag[1:].isdigit():
                problem.difficulty = Codeforces.normalize_difficulty(int(tag[1:]))
            else:
                problem.tags.append(tag)

        problem.statement = to_markdown(problem.statement)
        problem.testcases_sample_note = to_markdown(problem.testcases_sample_note)
        problem.input_format = to_markdown(problem.input_format)
        problem.output_format = to_markdown(problem.output_format)
        problem.constraints = 'See inline constraints in the input description'

        # print(problem.name, problem.limit_time, problem.limit_memory, problem.input_type,
        #       problem.output_type, problem.src_status_url, sep=' - ')
        CLog.info("Problem name: " + problem.name)
        # print('=====')
        # print(problem.statement)
        # print('=====')
        # print(problem.input_format)
        # print('=====')
        # print(problem.output_format)
        # print('=====')
        # print(problem.tags)
        # print('=====')
        # print(problem.difficulty)
        # print('=====')
        # for t in problem.testcases:
        #     print(t)
        #     print('----')

        submissions_cpp = self.get_submission_list(problem, 'cpp.g++11')
        submissions_fpc = self.get_submission_list(problem, 'pas.fpc')
        submissions_py3 = self.get_submission_list(problem, 'python.3')

        problem.stock_testcases_sample = problem.stock_testcases
        problem.stock_testcases = []
        CLog.info("Getting best solutions...")
        CLog.info("    C++ solutions:     %s" % len(submissions_cpp))
        CLog.info("    FPC solutions:     %s" % len(submissions_fpc))
        CLog.info("    Python3 solutions: %s" % len(submissions_py3))
        if submissions_cpp:
            solution_cpp, testcases = self.get_solution(submissions_cpp[0], True)
            problem.solutions.append({'lang': '11.cpp', 'code': solution_cpp})
            for testcase in testcases:
                problem.stock_testcases.append(testcase)
            CLog.info("Getting all visible testcases...")
            CLog.info("    Total visible testcases: %s" % len(problem.stock_testcases))
            CLog.info("    Total sample testcases:  %s" % len(problem.stock_testcases_sample))
        if submissions_fpc:
            solution_fpc, tmp = self.get_solution(submissions_fpc[0])
            problem.solutions.append({'lang':'pas', 'code': solution_fpc})
        if submissions_py3:
            submissions_py3, tmp = self.get_solution(submissions_py3[0])
            problem.solutions.append({'lang':'py', 'code': submissions_py3})

        # print("Solutions:", json.dumps(problem.solutions))

        # save_problem(dsa_problem, '../problems')

        return problem

    def get_submission_list(self, problem, language='anyProgramTypeForInvoker'):
        if not problem.src_status_url:
            return []
        frame_problem_index = problem.src_status_url[problem.src_status_url.rfind('/') + 1:]
        url = problem.src_status_url + "?order=BY_CONSUMED_TIME_ASC"

        headers = copy.deepcopy(self._headers)

        if not self.csrf:
            self.get_csrf()

        data = {
            'csrf_token': self.csrf,
            'frameProblemIndex': frame_problem_index,
            'action': 'setupSubmissionFilter',
            'comparisonType': 'NOT_USED',
            'verdictName': "OK",
            'programTypeForInvoker': language,
            # 'programTypeForInvoker': "anyProgramTypeForInvoker",
            # 'programTypeForInvoker': "pas.fpc",
            # 'programTypeForInvoker': "python.3",
            # 'programTypeForInvoker': "cpp.g++11",
            # 'programTypeForInvoker': "cpp.g++14",
            # 'programTypeForInvoker': "cpp.g++17",
            # 'programTypeForInvoker': "java8",
            # '_tta': 248
        }
        # print(data)
        r = self.s.post(url, headers=headers, data=data)
        #
        # print(r.status_code)
        # print(r.text)
        #
        # r = self.s.get(problem.src_status_url, headers=headers)

        raw = r.text
        # print(raw)

        soup = BeautifulSoup(raw, 'html.parser')

        submissions = soup.select('table.status-frame-datatable > tr[data-submission-id]')
        # print(submissions)
        # print(len(submissions))
        # problem.solutions
        result = []
        for submit in submissions:
            submission_url = 'https://codeforces.com/' + submit.select('td')[0].select('a')[0]['href']
            submission_id = submit.select('td')[0].select('a')[0].text
            submission_problem = submit.select('td')[3].select('a')[0]['href']
            submission_language = submit.select('td')[4].text.strip()
            submission_status = submit.select('td')[5].select('span > span')[0].text.strip()
            submission_time = submit.select('td')[6].text.strip().replace('\xa0', ' ')
            submission_memory = submit.select('td')[7].text.strip().replace('\xa0', ' ')
            submission = {
                'url': submission_url,
                'id': submission_id,
                'problem': submission_problem,
                'lang': submission_language,
                'verdict': submission_language,
                'status': submission_status,
                'time': submission_time,
                'memory': submission_memory,
            }
            # print(submission)
            result.append(submission)
            # problem.solutions.append(submission)
        return result

    def get_solution(self, submission, get_test_cases=False):
        # submission_url = submission['url']

        headers = copy.deepcopy(self._headers)
        # r = self.s.get(submission_url, headers=headers)
        # raw = r.text
        # soup = BeautifulSoup(raw, 'html.parser')
        # print(raw)
        # source_code = soup.select('pre#program-source-text')[0].text
        # print(source_code)

        r = self.s.post("https://codeforces.com/data/submitSource", headers=headers,
                       data={
                           'submissionId': submission['id'],
                           'csrf_token': self.csrf
                       })
        res = r.json()
        source_code = res['source']

        if not get_test_cases:
            return source_code, []

        # print(testcases)
        tesst_count = int(res['testCount'])
        testcases = []
        for t in range(tesst_count):
            input_data = res['input#%s' % (t + 1)]
            output_data = res['answer#%s' % (t + 1)]
            if input_data.endswith('...') or output_data.endswith('...'):  # uncompleted test data
                continue
            testcases.append(TestCase(input_data.strip(), output_data.strip(), name=str(t + 1)))
            # print(testcases[-1])

        return source_code, testcases

    @staticmethod
    def parse(url):
        """
        For problem: https://codeforces.com/problemset/problem/1257/D, the url can be:
        - https://codeforces.com/problemset/problem/1257/D
        - http://codeforces.com/problemset/problem/1257/D
        - codeforces.com/problemset/problem/1257/D
        - 1257/D
        - 1257D
        :param url:
        :return:
        - refined url: https://codeforces.com/problemset/problem/1257/D
        - status url: https://codeforces.com/problemset/status/1257/problem/D
        """
        url = url.strip()
        status_url = url
        problem_id = ''
        if url.startswith('https://') or url.startswith('http://'):
            problem_id = url[url[:-2].rfind('/')+1:]
        elif url.startswith('codeforces.com'):
            url = "https://" + url
            problem_id = url[url[:-2].rindex('/') + 1:]
        else:
            problem_id = url
            if url.find('/')<0:
                problem_id = problem_id[:-1] + "/" + problem_id[-1]
            url = "https://codeforces.com/problemset/problem/" + problem_id

        status_url = "https://codeforces.com/problemset/status/%s/problem/%s" % (problem_id[:-2], problem_id[-1:])
        return url, status_url

    @staticmethod
    def normalize_difficulty(original_difficulty):
        difficulty = original_difficulty ** 0.5
        dmax = 3800 ** 0.5
        dmin = 250 ** 0.5

        return (difficulty - dmin) / (dmax - dmin) * 10

    @staticmethod
    def original_difficulty(normalized_difficulty):
        normalized_difficulties = {Codeforces.normalize_difficulty(d): d for d in range(100, 4001, 100)}
        for k, v in normalized_difficulties.items():
            if abs(k-normalized_difficulty) < 0.001:
                return v

    def get_csrf(self):
        headers = copy.deepcopy(self._headers)
        r = self.s.get("https://codeforces.com/", headers=headers)
        raw = r.text
        soup = BeautifulSoup(raw, 'html.parser')
        self.csrf = soup.select('span.csrf-token')[0]['data-csrf']
        # print('CSRF:', self.csrf)


if __name__ == '__main__':
    cf = Codeforces()
    # problems, page_count = cf.get_problem_list(1)
    # CLog.info("Number of pages: %s" % page_count)
    #
    # for i in range(len(problems)):
    #     tags = ', '.join(problems[i].tags)
    #     print(f'#{i+1} - {problems[i].src_id} - {problems[i].name} - {problems[i].difficulty}'
    #           f' - {problems[i].src_url} - {tags}')
    #     break
    #
    # dsa_problem = cf.get_problem_detail(problems[4])

    # dsa_problem = cf.get_problem_from_url("https://codeforces.com/problemset/problem/1257/C")
    dsa_problem = cf.get_problem_from_url("1268/B")
    DsaProblem.save(dsa_problem, "../problems", overwrite=True)
    # print(dsa_problem)




import copy
import html

import requests
from tomd import Tomd
from bs4 import BeautifulSoup

from ptoolbox.dsa.dsa_problem import DsaProblem

from ptoolbox.models.general_models import Problem, TestCase


def to_markdown(html_text):
    if not html_text:
        return ""
    # html_text = html_text.replace("$$$", "$")
    soup = BeautifulSoup(html_text, 'html.parser')

    # img tag must be inside <p> tag
    imgs = soup.select("img")
    if imgs:
        for img in imgs:
            img['src'] = "https://open.kattis.com" + img['src']
            if "alt" in img:
                img.string = img['alt']
            del img['style']
            del img['alt']
            new_tag = soup.new_tag('p')
            img.wrap(new_tag)

    html_ = html.unescape(soup.decode())
    res = Tomd(html_).markdown.strip()

    return res


class Kattis:
    def __init__(self):
        self.username = None
        self.s = requests.session()
        self._headers = {
            'origin': 'https://open.kattis.com/',
            'referer': 'https://open.kattis.com/',
            'user-agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) '
                          'Chrome/70.0.3538.77 Safari/537.36',
        }

    def get_problem(self, problem_slug):
        if problem_slug.count('/') < 2:
            url = 'https://open.kattis.com/problems/{}'.format(problem_slug)
        else:
            problem_slug = problem_slug.strip("/")
            problem_slug = problem_slug[problem_slug.rfind("/") + 1:]
            url = 'https://open.kattis.com/problems/{}'.format(problem_slug)

        print("Getting problem from:", url)

        headers = copy.deepcopy(self._headers)
        r = self.s.get(url, headers=headers)

        raw = r.text

        soup = BeautifulSoup(raw, 'html.parser')
        problem_wrapper = soup.select('div.problem-wrapper')[0]
        problem_info = soup.select('div.problem-sidebar div.sidebar-info p')

        problem = Problem()
        if len(problem_info) > 3:
            difficulty = float(problem_info[3].select("span")[0].text)
            # print(difficulty)
            problem.difficulty = Kattis.normalize_difficulty(difficulty)
        problem.src_url = url
        problem.src_id = problem.slug = problem_slug
        problem.name = problem_wrapper.select("div.headline-wrapper > h1")[0].text
        print(problem.name)

        problem_body = problem_wrapper.select("div.problembody")[0]

        h_tags = problem_body.select("h2")
        if not h_tags:
            h_tags = problem_body.select("h3")
        input_tag = output_tag = None
        for h2 in h_tags:
            if h2.text == "Input":
                input_tag = h2
            elif h2.text == "Output":
                output_tag = h2

        # print(str(input_tag))

        # print(problem_wrapper.prettify())

        next_tag = input_tag.next_sibling
        input_tag.extract()
        input_tags = BeautifulSoup("", 'html.parser')
        while next_tag != output_tag:
            if next_tag:
                input_tags.append(copy.copy(next_tag))
            # print(next_tag)
            tmp = next_tag
            next_tag = next_tag.next_sibling
            tmp.extract()

        # print(input_tags.prettify())

        # print(str(output_tag))

        next_tag = output_tag.next_sibling
        output_tag.extract()
        output_tags = BeautifulSoup("", 'html.parser')
        first_sample = problem_body.select("table.sample")[0]
        while next_tag != first_sample:
            if next_tag:
                output_tags.append(copy.copy(next_tag))
            # print(next_tag)
            tmp = next_tag
            next_tag = next_tag.next_sibling
            tmp.extract()

        # print(output_tags.prettify())

        problem.input_format = to_markdown(input_tags.decode())
        problem.output_format = to_markdown(output_tags.decode())

        # print(problem.input_format)
        # print(problem.output_format)

        problem.constraints = 'See inline constraints in the input description'

        sample_tags = problem_body.select("table.sample")
        i = 0
        for sample_tag in sample_tags:
            # print(str(sample_tag))
            sample_test = sample_tag.select("tr > td > pre")
            testcase = TestCase(sample_test[0].text.strip(), sample_test[1].text.strip(), name=str(i + 1))
            # print(testcase)
            problem.stock_testcases_sample.append(testcase)
            sample_tag.extract()
            i += 1

        # print(len(problem.stock_testcases_sample))

        # print(str(problem_body))

        problem.statement = to_markdown(problem_body.decode())
        print(problem.statement)

        return problem

    @staticmethod
    def normalize_difficulty(original_difficulty):
        """
        [1.1, 8.5] --> [2.5, 10]
        :param original_difficulty:
        :return:
        """

        return original_difficulty + 1.5

    @staticmethod
    def original_difficulty(normalized_difficulty):
        return round(normalized_difficulty - 1.5, 1)


if __name__ == "__main__":
    # prepare_testcases("../problems/prob1", True)
    kt = Kattis()
    # kt.login('thucngch', '15041985')
    # dsa_problem = kt.get_problem("tarifa")
    dsa_problem = kt.get_problem("hittingtargets")

    DsaProblem.save(dsa_problem, "../problems", overwrite=True)




import os

import click

from ptoolbox.codeforces.codeforces import Codeforces
from ptoolbox.dsa import dsa_problem_file
from ptoolbox.dsa.dsa_problem import DsaProblem
from ptoolbox.hackerrank import hackerrank
from ptoolbox.hackerrank.hackerrank import HackerRank
from ptoolbox.helpers.clog import CLog
from ptoolbox.helpers.dsa_helper import read_all_problems
from ptoolbox.kattis.kattis import Kattis
from ptoolbox.ucode.ucode import UCode


@click.group()
def cli():
    """
    Problem Toolbox - Problem CLI tools by Thuc Nguyen (https://github.com/thucnc)
    """
    click.echo("Problem Toolbox - Problem CLI tools by Thuc Nguyen (https://github.com/thucnc)")


@cli.group(name='ucode')
def ucode_group():
    """
    ucode problem tools
    """
    click.echo("ucode problem tools")


@cli.group(name='dsa')
def dsa_group():
    """
    DSA problem tools
    """
    click.echo("Common DSA tools")


@cli.group(name='cf')
def codeforces_group():
    """
    Codeforces tools
    """
    click.echo("codeforces.com tools")


@cli.group(name='hr')
def hackerrank_group():
    """
    Hackerrank tools
    """
    click.echo("hackerrank.com tools")


@cli.group(name='kt')
def kattis_group():
    """
    Kattis tools
    """
    click.echo("kattis.com tools")


@dsa_group.command(name='create')
@click.option('-d', '--dir', default='.',
              type=click.Path(file_okay=False),
              prompt='Base directory for the problem', help='Base folder for the problem')
@click.option('--overwrite/--no-overwrite', default=False, help='Overwrite existing folder, default - No')
@click.argument('problem', metavar='{problem}')
def create_problem(dir, overwrite, problem):
    """
    Create a problem boilerplate

    Syntax:
    ptoolbox dsa create-problem -d {folder} {problem-code} [--overwrite]

    Ex.:
    ptoolbox dsa create-problem -d problems/ prob2 --overwrite

    """
    dsa_problem_file.create_problem(dir, problem, overwrite=overwrite)


@dsa_group.command(name='check')
@click.option('--autofix/--no-auto-fix', default=False, help='Auto fix style, save original file to .bak.md, default - No')
@click.argument('problem_folder', metavar='{problem_folder}')
def check_problem(autofix, problem_folder):
    """
    Check problem folder for proper format

    Syntax:
    ptoolbox dsa check-problem {problem-folder}

    Ex.:
    ptoolbox dsa check-problem ../problems/prob2

    """
    dsa_problem_file.check_problem(problem_folder, autofix)


@codeforces_group.command(name='get')
@click.option('-d', '--dir', default='.',
              type=click.Path(file_okay=False),
              prompt='Base directory for the problem', help='Base folder for the problem')
@click.option('-c', '--code', default='',
              prompt='Problem code, leave empty by default for auto generation from problem name',
              help='Problem code, use as problem folder name')
@click.argument('url', metavar='{problem_url-or-problem_id}')
def get_codeforces_problem(dir, code, url):
    """
    Get a codeforces problem and save as to a local folder

    Syntax:
    ptoolbox cf get [-d {folder}] [-c {problem-code}] {codeforces-problem-url-or-id}

    Ex.:
    ptoolbox cf get -d problems/ https://codeforces.com/problemset/problem/1257/D
    or:
    ptoolbox cf get -d problems/ -c "Monster Killing" 1257D
    or:
    ptoolbox cf get -d problems/ 1257/D

    """
    cf = Codeforces()
    dsa_problem = cf.get_problem_from_url(url)
    DsaProblem.save(dsa_problem, dir, code, overwrite=False)


@kattis_group.command(name='get')
@click.option('-d', '--dir', default='.',
              type=click.Path(file_okay=False),
              prompt='Base directory for the problem', help='Base folder for the problem')
@click.option('-c', '--code', default='',
              prompt='Problem code, leave empty by default for auto generation from problem name',
              help='Problem code, use as problem folder name')
@click.argument('slug', metavar='{kattis-problem_slug}')
def get_kattis_problem(dir, code, slug):
    """
    Get a codeforces problem and save as to a local folder

    Syntax:
    ptoolbox cf get [-d {folder}] [-c {problem-code}] {kattis-problem-id}

    Ex.:
    ptoolbox cf get -d problems/ -c "Monster Killing" pieceofcake2
    """
    kt = Kattis()
    dsa_problem = kt.get_problem(slug)
    DsaProblem.save(dsa_problem, dir, code, overwrite=False)


@hackerrank_group.command()
@click.option('--keep-zip-file-only/--keep-intermediate-files', default=True,
              help='Remove intermediate files, default - Yes')
@click.argument('problem_folder', metavar='{problem_folder}')
def prepare_testcases(keep_zip_file_only, problem_folder):
    """
    Convert testcases to hackerrank format, and compress into .zip file, ready for upload

    Syntax:
    ptoolbox hackerrank prepare-testcases  {problem-folder} [--keep-zip-file-only/--keep-intermediate-files]

    Ex.:
    ptoolbox hackerrank prepare-testcases problems/prob2

    """
    hackerrank.prepare_testcases(problem_folder, keep_zip_file_only=keep_zip_file_only)


@hackerrank_group.command(name="create")
@click.option('-c', '--credential', default='credentials.ini',
              type=click.Path(dir_okay=False, exists=True),
              prompt='Credential file', help='Configuration file that contain hackerrank user name/pass')
@click.option('--with-testcases/--without-testcases', default=True, help='Upload testcases')
@click.option('-w', '--weight', default=100, type=click.INT, help='Weight (score) of each testcase')
@click.option('-s', '--sample', default=1, type=click.INT, help='Number of sample testcases')
@click.argument('problem_folder', metavar='{problem_folder}')
def create_problem(credential, with_testcases, weight, sample, problem_folder):
    """
    Create problem description on hackerrank

    Syntax:
    ptoolbox hackerrank create-problem [--upload-testcases] {dsa_problem_folder}

    Ex.:
    ptoolbox hackerrank create-problem problems/array001_counting_sort3/

    """
    username, password = HackerRank.read_credential(credential)
    if not username or not password:
        CLog.error(f'Username and/or password are missing in {credential} file')
        return

    hr = HackerRank()
    hr.login(username, password)

    problem1 = DsaProblem.load(problem_folder)
    hk_problem = hr.create_problem(problem1)

    if hk_problem and with_testcases:
        _do_upload_testcases(username, password, weight, sample, hk_problem["id"], problem_folder)

    # print(hk_problem)

    if problem1:
        CLog.important(f'Problem `{hk_problem["id"]}` updated, slug: {hk_problem["slug"]}')


@hackerrank_group.command(name="update")
@click.option('-c', '--credential', default='credentials.ini',
              type=click.Path(dir_okay=False, exists=True),
              prompt='Credential file', help='Configuration file that contain hackerrank user name/pass')
@click.argument('hackerrank_problem_id', metavar='{hackerrank_problem_id}')
@click.argument('problem_folder', metavar='{problem_folder}')
def update_problem(credential, hackerrank_problem_id, problem_folder):
    """
    Update problem description on hackerrank

    Syntax:
    ptoolbox hackerrank update-problem {hackerrank_problem_id} {dsa_problem_folder}

    Ex.:
    ptoolbox hackerrank update-problem 113357 problems/array001_counting_sort3/

    """
    username, password = HackerRank.read_credential(credential)
    if not username or not password:
        CLog.error(f'Username and/or password are missing in {credential} file')
        return

    hr = HackerRank()
    hr.login(username, password)

    problem1 = DsaProblem.load(problem_folder)

    hk_problem = hr.update_problem(hackerrank_problem_id, problem1)

    # print(hk_problem)

    if problem1:
        CLog.important(f'Problem `{hk_problem["id"]}` updated, slug: `{hk_problem["slug"]}`')


def _do_upload_testcases(username, password, weight, sample, hackerrank_problem_id, problem_folder):
    testcase_zip = os.path.join(problem_folder, 'testcases_hackerrank.zip')

    if not os.path.exists(testcase_zip):
        CLog.error(f'`{testcase_zip}` does not exist, '
                   f'please run `ptoolbox hackerrank prepare-testcases {problem_folder}` first')
        return

    hr = HackerRank()
    hr.login(username, password)
    created_testcases = hr.upload_testcases_and_set_score(hackerrank_problem_id, testcase_zip, weight, sample)

    print(created_testcases)

    if created_testcases:
        CLog.important(f'{len(created_testcases)} test cases uploaded to problem `{hackerrank_problem_id}`')


@hackerrank_group.command()
@click.option('-c', '--credential', default='credentials.ini',
              type=click.Path(dir_okay=False, exists=True),
              prompt='Credential file', help='Configuration file that contain hackerrank user name/pass')
@click.option('-w', '--weight', default=100, type=click.INT, help='Weight (score) of each testcase')
@click.option('-s', '--sample', default=1, type=click.INT, help='Number of sample testcases')
@click.argument('hackerrank_problem_id', metavar='{hackerrank_problem_id}')
@click.argument('problem_folder', metavar='{problem_folder}')
def upload_testcases(credential, weight, sample, hackerrank_problem_id, problem_folder):
    """
    Upload testcases_hackerrank.zip to problem on hackerrank

    Syntax:
    ptoolbox hackerrank upload-testcases -w {weight} -s {sample_count} {hackerrank_problem_id} {dsa_problem_folder}

    Ex.:
    ptoolbox hackerrank upload-testcases -w 100 -s 2 113357 problems/array001_counting_sort3/

    """
    username, password = HackerRank.read_credential(credential)
    if not username or not password:
        CLog.error(f'Username and/or password are missing in {credential} file')
        return

    _do_upload_testcases(username, password, weight, sample, hackerrank_problem_id, problem_folder)


@ucode_group.command(name="create")
@click.option('-c', '--credential', default='credentials.ini',
              type=click.Path(dir_okay=False, exists=True),
              prompt='Credential file', help='Configuration file that contain hackerrank user name/pass')
@click.option('-l', '--lesson', type=click.INT, help='Lesson id')
@click.option('-s', '--score', default=100, type=click.INT, help='Score of this question')
@click.option('-x', '--xp', default=100, type=click.INT, help='XP of this question')
@click.argument('problem_folder', metavar='{problem_folder}')
def ucode_create_problem(lesson, credential, score, problem_folder, xp):
    """
    Create problem on ucode.vn

    Syntax:
    ptoolbox ucode create-problem [--upload-testcases] -l {lesson_id} -s {score} {dsa_problem_folder}

    """
    api_url, token = UCode.read_credential(credential)
    if not api_url or not token:
        CLog.error(f'Username and/or password are missing in {credential} file')
        return

    ucode = UCode(api_url, token)

    problem = DsaProblem.load(problem_folder, load_testcase=True)
    problem_id = ucode.create_problem(lesson, problem, score=score, xp=xp)

    if problem:
        CLog.important(f'Problem `{problem_id}` created')


@ucode_group.command(name="create-multiple")
@click.option('-c', '--credential', default='credentials.ini',
              type=click.Path(dir_okay=False, exists=True),
              prompt='Credential file', help='Configuration file that contain hackerrank user name/pass')
@click.option('-l', '--lesson', type=click.INT, help='Lesson id')
@click.option('-s', '--score', default=100, type=click.INT, help='Score of this question')
@click.option('-x', '--xp', default=100, type=click.INT, help='XP of this question')
@click.argument('base_folder', metavar='{base_folder}')
def ucode_create_problems(lesson, credential, score, base_folder, xp):
    """
    Create multiple problems on ucode.vn

    Syntax:
    ptoolbox ucode create-problem [--upload-testcases] -l {lesson_id} -s {score} {dsa_problem_folder}

    """
    api_url, token = UCode.read_credential(credential)
    if not api_url or not token:
        CLog.error(f'Username and/or password are missing in {credential} file')
        return

    ucode = UCode(api_url, token)

    problems = read_all_problems(base_folder, load_testcase=True)

    for problem_tuple in problems:
        problem = problem_tuple[1]
        problem_id = ucode.create_problem(lesson, problem, score=score, xp=xp)
        if problem:
            CLog.important(f'Problem `{problem_id}` created')


import copy
import html

import requests
from tomd import Tomd
from bs4 import BeautifulSoup

from ptoolbox.models.general_models import Problem, TestCase


def to_markdown(html_text):
    if not html_text:
        return ""
    # html_text = html_text.replace("$$$", "$")
    soup = BeautifulSoup(html_text, 'html.parser')

    # img tag must be inside <p> tag
    imgs = soup.select("center > img")
    if imgs:
        for img in imgs:
            img['src'] = "https://open.kattis.com" + img['src']
            img.string = img['alt']
            del img['style']
            del img['alt']
            img.parent.name = "p"

    html_ = html.unescape(soup.decode())
    print(html_)
    res = Tomd(html_).markdown.strip()

    return res


class GeeksForGeeks:
    def __init__(self):
        self.username = None
        self.s = requests.session()
        self._headers = {
            'origin': 'https://practice.geeksforgeeks.org/',
            'referer': 'https://practice.geeksforgeeks.org/',
            'user-agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) '
                          'Chrome/70.0.3538.77 Safari/537.36',
        }

    def get_problem(self, problem_slug):
        url = 'https://practice.geeksforgeeks.org/problems/{}'.format(problem_slug)
        headers = copy.deepcopy(self._headers)
        r = self.s.get(url, headers=headers)

        raw = r.text

        soup = BeautifulSoup(raw, 'html.parser')
        problem_title_tag = soup.select('div#border div.row strong')[0]

        problem = Problem()
        problem.src_url = url
        problem.src_id = problem.slug = problem_slug
        problem.name = problem_title_tag.text.strip()
        print(problem.name)

        return

        problem_body = soup.select("div.problemQuestion")[0]
        h2s = problem_body.select("h2")
        input_tag = output_tag = None
        for h2 in h2s:
            if h2.text == "Input":
                input_tag = h2
            elif h2.text == "Output":
                output_tag = h2

        print(str(input_tag))

        # print(problem_wrapper.prettify())

        next_tag = input_tag.next_sibling
        input_tag.extract()
        input_tags = BeautifulSoup("", 'html.parser')
        while next_tag != output_tag:
            if next_tag:
                input_tags.append(copy.copy(next_tag))
            # print(next_tag)
            tmp = next_tag
            next_tag = next_tag.next_sibling
            tmp.extract()

        print(input_tags.prettify())

        print(str(output_tag))

        next_tag = output_tag.next_sibling
        output_tag.extract()
        output_tags = BeautifulSoup("", 'html.parser')
        first_sample = problem_body.select("table.sample")[0]
        while next_tag != first_sample:
            if next_tag:
                output_tags.append(copy.copy(next_tag))
            # print(next_tag)
            tmp = next_tag
            next_tag = next_tag.next_sibling
            tmp.extract()

        print(output_tags.prettify())

        problem.input_format = to_markdown(input_tags.decode())
        problem.output_format = to_markdown(output_tags.decode())

        print(problem.input_format)
        print(problem.output_format)

        problem.constraints = 'See inline constraints in the input description'

        sample_tags = problem_body.select("table.sample")
        i = 0
        for sample_tag in sample_tags:
            # print(str(sample_tag))
            sample_test = sample_tag.select("tr > td > pre")
            testcase = TestCase(sample_test[0].text.strip(), sample_test[1].text.strip(), name=str(i + 1))
            # print(testcase)
            problem.stock_testcases_sample.append(testcase)
            sample_tag.extract()
            i += 1

        print(len(problem.stock_testcases_sample))

        print(str(problem_body))

        problem.statement = to_markdown(problem_body.decode())
        print(problem.statement)

        return problem


if __name__ == "__main__":
    # prepare_testcases("../problems/prob1", True)
    gg = GeeksForGeeks()
    # dsa_problem = gg.get_problem("counts-zeros-xor-pairs/0")
    # dsa_problem = gg.get_problem("n-queen-problem/0")

    problem_slugs = [
        'n-queen-problem/0',
        'counts-zeros-xor-pairs/0',
        'rotate-array-by-n-elements/0',
        'kth-smallest-element/0',
        'arraylist-operation/1',
        'max-distance-between-same-elements/1',
        'game-of-death-in-a-circle/0',
        'diagonal-sum-in-binary-tree/1',
        'delete-middle-element-of-a-stack/1',
        'next-happy-number/0',
        'count-subsequences-of-type-ai-bj-ck4425/1',
        'count-subsequences-of-type-ai-bj-ck/0',
        'distinct-occurrences/1',
        'count-of-strings-that-can-be-formed-using-a-b-and-c-under-given-constraints/0',
    ]

    for slug in problem_slugs:
        dsa_problem = gg.get_problem(slug)
        break

    # DsaProblem.save(dsa_problem, "../problems", overwrite=True)




import copy
import json
import traceback
from datetime import datetime

from dateutil import tz, parser

from draftjs_exporter.html import HTML
from draftjs_exporter_markdown import BLOCK_MAP, ENGINE, ENTITY_DECORATORS, STYLE_MAP

import requests

from ptoolbox.dsa.dsa_problem import DsaProblem
from ptoolbox.models.general_models import MatchingType, Classroom, ProgrammingLanguage, ProblemStatus, TestCase, \
    JudgeMode, Problem


class ReplIt:
    def __init__(self, cookies):
        self.s = requests.session()
        self._headers = {
            'origin': 'https://repl.it',
            'referer': 'https://repl.it/login',
            'user-agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.77 Safari/537.36',
            'cookie': cookies
        }
        self.cookies = cookies

    def get_classroom(self, classroom_id, get_assignments=True, get_students=False, get_submissions=False,
                      get_all_problems=True):
        url = 'https://repl.it/data/classrooms/{}'.format(classroom_id)
        headers = copy.deepcopy(self._headers)
        r = self.s.get(url, headers=headers)
        raw = r.json()

        classroom = Classroom()
        classroom.src_id = raw['id']
        classroom.name = raw['name']
        if raw['language_key'] == 'python3':
            classroom.language = ProgrammingLanguage.python3

        problems = []
        if get_assignments:
            url = 'https://repl.it/data/classrooms/{}/assignments'.format(classroom_id)
            r = self.s.get(url, headers=headers)
            assignments = r.json()
            for a in assignments:
                for p in ['classroom_id', 'time_deleted', 'time_updated', 'time_due', 'time_created', 'github_id']:
                    a.pop(p, None)
                # time_published = null: draft
                # time_published > now: scheduled
                # time_published <= now: published
                if not a['time_published']:
                    a['status'] = ProblemStatus.draft
                else:
                    now = datetime.now()
                    # from_zone = tz.tzutc()
                    to_zone = tz.tzlocal()
                    published_date = parser.parse(a['time_published'])
                    # published_date = published_date.replace(tzinfo=from_zone)
                    local = published_date.astimezone(to_zone).replace(tzinfo=None)
                    if local < now:
                        a['status'] = ProblemStatus.published
                    else:
                        a['status'] = ProblemStatus.scheduled

            classroom.assignments = sorted(assignments, key=lambda x: x['name'])
            if get_all_problems:
                # classroom
                problems = self.get_problems([asm['id'] for asm in assignments])

        if get_students:
            url = 'https://repl.it/data/classrooms/{}/students'.format(classroom_id)
            # r = self.s.get(url, headers=headers)
            # students = r.json()
            # TODO
        if get_submissions:
            url = 'https://repl.it/data/classrooms/{}/submissions'.format(classroom_id)
            # r = self.s.get(url, headers=headers)
            # students = r.json()
            # TODO

        return classroom, problems

    def get_problems(self, problem_ids):
        """
        :param problem_ids: list of problem ids
        :return:
        """
        problems = []
        for id in problem_ids:
            problems.append(self.get_problem(id))
        return problems

    def get_problem(self, problem_id):
        url = 'https://repl.it/data/assignments/{}'.format(problem_id)
        headers = copy.deepcopy(self._headers)
        r = self.s.get(url, headers=headers)

        print(r.text)

        raw = r.json()
        problem = self.parse_problem(raw)
        return problem

    def get_testcases(self, problem_id):
        url = 'https://repl.it/data/assignments/{}/tests/'.format(problem_id)
        r = self.s.get(url, headers=self._headers)
        raw = r.json()
        print(raw)

        testcases = []
        for t in raw:
            testcase = self.parse_testcase(t)
            testcases.append(testcase)

        return testcases

    def get_solution(self, problem_id):
        url = 'https://repl.it/data/teacher/model_solutions/{}'.format(problem_id)
        r = self.s.get(url, headers=self._headers)
        raw = r.json()
        if 'editor_text' in raw:
            sol = raw['editor_text']
        else:
            sol = ''
        return sol

    def parse_problem(self, raw):
        exporter = HTML({
            'block_map': BLOCK_MAP,
            'style_map': STYLE_MAP,
            'entity_decorators': ENTITY_DECORATORS,
            'engine': ENGINE,
        })

        if not raw['instructions']:
            markdown = ''
        else:
            try:
                markdown = exporter.render(json.loads(raw['instructions']))
            except Exception:
                markdown = ""
                traceback.print_exc()

        problem = Problem()
        problem.statement = markdown

        if raw['feedback_mode'] == "input_output":
            problem.judge_mode = JudgeMode.oj
        elif raw['feedback_mode'] == "manual":
            problem.judge_mode = JudgeMode.manual
        else:
            problem.judge_mode = JudgeMode.unit_test

        problem.src_id = raw['id']
        problem.name = raw['name']
        problem.template = raw['editor_template']

        # print(markdown)

        problem.testcases = self.get_testcases(problem.src_id)
        # print(problem.testcases)

        if problem.testcases:
            problem.testcases_sample = problem.testcases[:1]

        problem.solution = self.get_solution(problem.src_id)
        return problem

    def parse_testcase(self, t):
        if t['matching_type'] == 'flexible':
            mt = MatchingType.flexible
        elif t['matching_type'] == 'strict':
            mt = MatchingType.strict
        else:
            mt = MatchingType.regexp
        testcase = TestCase(t['input'], t['output'], matchingtype=mt)
        testcase.src_id = t['id']
        return testcase


if __name__ == "__main__":
    cookies = \
        "_ga=GA1.2.1677725907.1574165625; ajs_group_id=null; " \
        "ajs_anonymous_id=%22ffa18d4a-381b-4ef8-91c8-8b8512a84015%22; " \
        "ajs_user_id=816683; " \
        "ajs_group_id=null; " \
        "ajs_anonymous_id=%22ffa18d4a-381b-4ef8-91c8-8b8512a84015%22; " \
        "_gid=GA1.2.1064340347.1586530886; " \
        "__cfduid=d71e0129e3276acbcf8a4c30508fa44591586598686; " \
        "connect.sid=s%3ABIXcPR3N1fqg6-Q95N431rGktezZS9tL.o6GU8PWzP4eLBvC%2Bgo5rczm4dVSwJwFhK%2FNaChC1Qns; " \
        "_gat=1; ajs_user_id=816683"
    repl = ReplIt(cookies)
    classroom, problems = repl.get_classroom(classroom_id=140651, get_assignments=True, get_all_problems=True)
    for problem in problems:
        DsaProblem.save(problem, "../../problems/lesson5")
    # print(classroom)
    # for assignment in classroom.assignments:
    #     print(assignment)

    # problem = repl.get_problem(5064610)
    # DsaProblem.save(problem, "../../problems/p1")


    # problem = repl.get_problem(2079412)
    # problem = repl.get_problem(2079413)
    # problem = repl.get_problem(2079416)
    # problem = repl.get_problem(2079417)

    # i = 1
    # for t in problem.testcases:
    #     print('#{}: '.format(i), t)
    #     i += 1

    # classroom = repl.get_classroom("85432")
    # print(classroom)
    # pprint.pprint(classroom.assignments)

    # test_create_problem()


import codecs
import glob
import os
import re
from shutil import copyfile
from jinja2 import Template

from ptoolbox.helpers.clog import CLog


def create_problem(folder, problem_code, overwrite=False):
    problem_code = problem_code.replace('-', '_').replace(' ', '_').lower()

    problem_folder = os.path.join(folder, problem_code)

    if os.path.exists(problem_folder):
        if not overwrite:
            CLog.error('Problem folder existed! Delete the folder or use `overwrite` instead.')
            return
        else:
            CLog.warn('Problem folder existed! Content will be overwritten.')

    if not os.path.exists(problem_folder):
        os.makedirs(problem_folder)

    # if not os.path.exists(problem_folder + "/testcases"):
    #     os.makedirs(problem_folder + "/testcases")

    problem_name = (' '.join(problem_code.split('_'))).title()

    template_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'templates')

    with open(os.path.join(template_path, 'statement.md')) as file_:
        template = Template(file_.read())
        statement = template.render(problem_name=problem_name, problem_code=problem_code)
        f = codecs.open(problem_folder + ("/%s.md" % problem_code), "w", "utf-8")
        f.write(statement)
        f.close()

    with open(os.path.join(template_path, 'testcases.txt')) as file_:
        template = Template(file_.read())
        content = template.render()
        f = open(problem_folder + "/testcases.txt", 'w')
        f.write(content)
        f.close()

    with open(os.path.join(template_path, 'testcases_manual.txt')) as file_:
        template = Template(file_.read())
        content = template.render()
        f = open(problem_folder + "/testcases_manual.txt", 'w')
        f.write(content)
        f.close()

    with open(os.path.join(template_path, 'solution.py.j2')) as file_:
        template = Template(file_.read())
        content = template.render(problem_code=problem_code)
        f = open(problem_folder + ("/%s.py" % problem_code), 'w')
        f.write(content)
        f.close()

    with open(os.path.join(template_path, 'generator.py.j2')) as file_:
        template = Template(file_.read())
        content = template.render(problem_code=problem_code)
        f = open(problem_folder + ("/%s_generator.py" % problem_code), 'w')
        f.write(content)
        f.close()

    problem_folder = os.path.abspath(problem_folder)

    CLog.important(f'Problem created at `{problem_folder}`')


def read_testcases_from_file(testcase_file):
    count = 0
    inputi = ""
    outputi = ""
    is_output = False

    testcases = []

    with open(testcase_file, 'r') as fi:
        for line in fi:
            if line.startswith("###"):

                if count > 0:
                    testcases.append({'input': inputi.strip(), 'output': outputi.strip()})

                count += 1

                is_output = False

                inputi = outputi = ""

                continue
            elif line.startswith("---"):
                is_output = True
            else:
                if is_output:
                    outputi += line
                else:
                    inputi += line
        if inputi.strip() or outputi.strip():
            testcases.append({'input': inputi.strip(), 'output': outputi.strip()})

    return testcases


def find_section(pattern, lines, start_index=0, once=False):
    indices = []
    content = {}
    for i in range(start_index, len(lines)):
        if re.match(pattern, lines[i], re.I):
            indices.append(i)
            for j in range(i + 1, len(lines)):
                if lines[j].startswith('#'):
                    break
                content.setdefault(i, [])
                content[i].append(lines[j])
            if once:
                break

    return indices, content


def fix_section_title(name, pattern, lines, replace, once=True):
    found = False
    for i in range(len(lines)):
        if re.match(pattern, lines[i], re.I):
            lines[i] = re.sub(pattern, replace, lines[i])
            CLog.info(f'AUTOFIX: Fix {name} heading style: ' + lines[i])
            found = True
            if once:
                break
    if not found:
        CLog.warn(f'AUTOFIX: {name} is probably missing, don\'t know how to fix it.')


def check_section(name, pattern, lines, proper_line, unique=True, start_index=0, log_error=True):
    lines, content = find_section(pattern, lines, start_index)
    if log_error:
        flog = CLog.error
    else:
        flog = CLog.warn
    if not lines:
        flog(f'{name} is missing or invalid: {name} should has style: `{proper_line}`')
        return None, None
    if unique and len(lines) > 1:
        CLog.error(f'Only one {name} allowed!')

    empty = True
    for isection in content:
        for line in content[isection]:
            if line.strip() and not line.startswith('[//]:'):
                empty = False
    if empty:
        flog(f'{name} is empty!')

    return lines[0], content


def check_problem(problem_folder, auto_fix=False):
    problem_code = os.path.basename(problem_folder.rstrip(os.path.sep))

    statement_file = os.path.join(problem_folder, f"{problem_code}.md")
    if not os.path.isfile(statement_file):
        statement_files = glob.glob(os.path.join(problem_folder, "*.md"))
        if len(statement_files) < 1:
            raise SyntaxError(f'Problem statement file `{problem_code}.md` is missing!')
        elif len(statement_files) == 1:
            statement_file = statement_files[0]
            problem_code = os.path.splitext(os.path.basename(statement_file))[0]
        else:
            bak_files = [f for f in glob.glob(os.path.join(problem_folder, "*.md"))
                         if ".vi." in f or "_editorial.md" in f or "_bak.md" in f or "_bak1.md" in f or "_bak2.md" in f]
            if len(statement_files) - 1 > len(bak_files):
                raise SyntaxError('Problem folder contains more than one .md files, '
                                  'and no files has same name as the containing folder, don\'t know what to do.')
            else:
                t = set(statement_files) - set(bak_files)
                statement_file = list(t)[0]
                problem_code = os.path.splitext(os.path.basename(statement_file))[0]
                CLog.info('Auto detect .md file: %s' % statement_file)

    solution_file = os.path.join(problem_folder, f"{problem_code}.py")
    test_generator_file = os.path.join(problem_folder, f"{problem_code}_generator.py")
    testcase_file = os.path.join(problem_folder, f"testcases.txt")
    testcase_manual_file = os.path.join(problem_folder, f"testcases_manual.txt")

    if not os.path.isfile(solution_file):
        CLog.error(f"Solution file `{problem_code}.py` is missing!")
    if not os.path.isfile(test_generator_file):
        CLog.error(f"Testcase generator file `{problem_code}_generator.py` is missing!")
    if not os.path.isfile(testcase_file):
        CLog.error(f"Testcases file `testcases.txt` is missing!")
    else:
        file_size = os.stat(testcase_file).st_size
        if file_size > 50 * 1024 * 1024:
            CLog.error(f"Testcases file `testcases.txt` should not be > 50MB!")

    if not os.path.isfile(testcase_manual_file):
        CLog.warn(f"Manual testcases file `testcases_manual.txt` is missing!")

    with open(statement_file) as fi:
        statement = fi.read()
        # print(statement)
        lines = statement.splitlines()

        if not lines[0].startswith('[//]: # ('):
            CLog.error('The first line should be the source of the problem, ex. `[//]: # (http://source.link)`')
            if auto_fix:
                i = 0
                while not lines[i].strip():
                    i += 1
                lines = lines[i:]
                if lines[0].startswith('[//]:'):
                    lines[0] = '[//]: # (%s)' % lines[0][5:].strip()
                    CLog.info("AUTOFIX: convert source info to: " + lines[0])
                else:
                    CLog.warn("AUTOFIX: Source info may be missed. Don't know how to fix it.")
                    lines.insert(0, '[//]: # ()')

        title_line, statement = check_section('Title', '# \S*', lines, '# Problem Title (heading 1)')
        if auto_fix and not title_line:
            heading_line, content = find_section(f'#.*{problem_code}', lines)
            proper_title = (' '.join(problem_code.split('_'))).title()
            if not heading_line:
                CLog.info('AUTOFIX: Problem title is probably missing, adding one...')
                title1 = f'# {proper_title}'
                title2 = f'[//]: # ({problem_code})'
                lines.insert(1, '')
                lines.insert(2, title1)
                lines.insert(3, title2)
                print(*lines[:4], sep='\n')
            else:  # has Title but wrong format
                lines[heading_line[0]] = f'# {proper_title}'
                if not lines[heading_line[0] + 1].startswith("[//]:"):
                    lines.insert(heading_line[0] + 1, f'[//]: # ({problem_code})')
                CLog.info('AUTOFIX: Fix Title style and problem code')

        if title_line:
            title = lines[title_line]
            proper_title = (' '.join(title.split('_'))).title()
            if title != proper_title:
                CLog.warn(f'Improper title: `{title}`, should be `{proper_title}`')
            # proper_problem_code = f'[//]: # ({problem_code})'
            if not lines[title_line + 1].startswith('[//]: # ('):
                CLog.error(f'Title should be followed by proper problem code: `problem_code`')

            if lines[title_line + 3].startswith("[//]:") and lines[0] == '[//]: # ()':
                CLog.warn('Detect source link in statement: %s' % lines[title_line + 3])
                if auto_fix:
                    CLog.info('AUTOFIX: Detect source link in statement: %s, moving to first line'
                              % lines[title_line + 3])
                    lines[0] = '[//]: # (%s)' % lines[title_line + 3][5:].strip()
                    lines.pop(title_line + 3)

        input_line, input = check_section('Input', '## Input\s*$', lines, '## Input')
        if input_line and title_line and input_line < title_line:
            CLog.error('Input should go after the Problem Statement.')

        if auto_fix and input_line is None:
            fix_section_title('Input', f'(#+\s*Input|Input\s*$)', lines, f'## Input')

        constraints_line, constraints = check_section('Constraints', '## Constraints\s*$',
                                                      lines, '## Constraints')
        if constraints_line and input_line and constraints_line < input_line:
            CLog.error('Constraints should go after the Input.')

        if auto_fix and constraints_line is None:
            fix_section_title('Constraints', f'(#+\s*Constraint.*|Constraints\s*$|#+\s*Giới hạn.*)', lines,
                              f'## Constraints')

        output_line, output = check_section('Output', '## Output\s*$', lines, '## Output')
        # if output_line and constraints_line and output_line < constraints_line:
        #     CLog.error('Output should go after the Constraints.')

        if auto_fix and output_line is None:
            fix_section_title('Output', f'(#+\s*Output.*|Output\s*$|#+\s*Ouput.*|Ouput\s*$)', lines, f'## Output')

        tag_line, tag = check_section('Tags', '## Tags\s*$', lines, '## Tags')
        if tag_line and output_line and tag_line < output_line:
            CLog.error('Tags should go after the Output.')

        if auto_fix and tag_line is None:
            fix_section_title('Tags', f'#+\s*Tag.*', lines, f'## Tags')

        difficulty_line, difficulty = check_section('Difficulty', '## Difficulty\s*$', lines, '## Difficulty')
        if difficulty_line:
            try:
                difficulty = float(difficulty[difficulty_line][0])
                if difficulty < 1 or difficulty > 10:
                    CLog.error('Difficulty should be a number between 1 and 10, found: ' + str(difficulty))
            except ValueError:
                CLog.error('Difficulty should be a number between 1 and 10, found: ' + difficulty[difficulty_line][0])

        if auto_fix and difficulty_line is None:
            fix_section_title('Difficulty', f'#+\s*Difficulty.*', lines, f'## Difficulty')

        list_lines, list_content = find_section('- .*', lines)
        for i in list_lines[::-1]:
            if i > 0:
                prev_line = lines[i - 1]
                if prev_line.strip() and not prev_line.startswith('- '):
                    CLog.error(f'There should be an empty line before the list, line {i}: {lines[i]}')
                    if auto_fix:
                        CLog.info('AUTOFIX: Added new line before list')
                        lines.insert(i, '')

        lines2, tmp = check_section('Sample input', '## Sample input', lines, '## Sample input 1', unique=False)
        if auto_fix and not lines2:
            fix_section_title('Sample Input', '#+\s*Sample input(.*)', lines, r'## Sample Input\1', once=False)

        lines2, tmp = check_section('Sample output', '## Sample output', lines, '## Sample output 1', unique=False)
        if auto_fix and not lines2:
            fix_section_title('Sample Output', '#+\s*Sample (ou|Ou|out|Out)put(.*)', lines, r'## Sample Output\2',
                              once=False)

        lines2, tmp = check_section('Explanation', '## Explanation', lines, '## Explanation 1',
                                    unique=False, log_error=False)
        if auto_fix and not lines2:
            fix_section_title('Explanation', '#+\s*Explanation(.*)', lines, r'## Explanation\1', once=False)

        if auto_fix:
            statement_file_bak = os.path.join(problem_folder, f"{problem_code}_bak.md")
            i = 1
            while os.path.exists(statement_file_bak):
                statement_file_bak = os.path.join(problem_folder, f"{problem_code}_bak{i}.md")
                i += 1
            copyfile(statement_file, statement_file_bak)
            with open(statement_file, 'w') as f:
                f.write('\n'.join(lines))
    return problem_code


if __name__ == '__main__':
    create_problem('../problems', 'Array001 Counting-Sort3', overwrite=True)

    # check_problem('/home/thuc/teko/online-judge/ptoolbox/problems/array001_counting_sort3')
    # check_problem('/home/thuc/teko/online-judge/dsa-problems/array1d/arr001_counting_sort', auto_fix=True)
    # check_problem('/home/thuc/teko/online-judge/dsa-problems/array1d/arr002_football', auto_fix=True)

    # print(re.sub('#+\s*Sample input(.*)', r'## Sample Input\1', '# Sample input 2'))

    # tcs = read_testcases_from_file('../problems/prob1/testcases.txt')
    # print(*tcs, sep='\n')



import os
import codecs
from jinja2 import Template
from unidecode import unidecode

from ptoolbox.dsa.dsa_problem_file import check_problem, find_section, read_testcases_from_file
from ptoolbox.helpers.clog import CLog
from ptoolbox.models.general_models import Problem, TestCase


def join_lines(lines):
    return ''.join([l+'\n' for l in lines]).strip()


class DsaProblem:
    @staticmethod
    def load(problem_folder, load_testcase=False):
        problem_code = check_problem(problem_folder)
        statement_file = os.path.join(problem_folder, f"{problem_code}.md")

        problem = Problem()
        problem.slug = problem_code.replace('_', '-')
        problem.code = problem_code
        problem.preview = f'[//]: # ({problem_code})'

        with open(statement_file) as fi:
            statement = fi.read()
            lines = statement.splitlines()

            source_link = None
            s = lines[0]
            if not s.strip():
                s = lines[1]
            if s.startswith('[//]:'):
                o = s.find('(')
                if o:
                    source_link = s[o + 1:s.find(')')].strip()
                else:
                    source_link = s[5:].strip()
            problem.src_url = source_link

            statement_i, statement_c = find_section('#\s+.*', lines)
            if statement_i:
                title = lines[statement_i[0]][1:].strip()
                problem.name = title

                s = lines[statement_i[0] + 1].strip()
                if not s.strip():
                    s = lines[statement_i[0] + 2]
                if s.startswith('[//]:'):
                    problem.preview = s
                    o = s.find('(')
                    if o:
                        code = s[o + 1:s.find(')')].strip()
                    else:
                        code = s[5:].strip()
                    problem.code = code
                    problem.slug = problem.code.replace('_', '-')

            if statement_i:
                problem.statement = join_lines(statement_c[statement_i[0]])

            input_i, input_c = find_section('(#+\s*Input|Input\s*$)', lines)
            if input_i:
                problem.input_format = join_lines(input_c[input_i[0]])

            output_i, output_c = find_section('(#+\s*Output.*|Output\s*$|#+\s*Ouput.*|Ouput\s*$)', lines)
            if output_i:
                problem.output_format = join_lines(output_c[output_i[0]])

            constraints_i, constraints_c = find_section('(#+\s*Constraint.*|Constraints\s*$|#+\s*Giới hạn.*)', lines)
            if constraints_i:
                problem.constraints = join_lines(constraints_c[constraints_i[0]])

            tags_i, tags_c = find_section('#+\s*Tag.*', lines)
            if tags_i:
                tags = []
                for t in tags_c[tags_i[0]]:
                    t = t.strip()
                    if t:
                        if t.startswith('-'):
                            tags.append(t[1:].strip())
                        else:
                            tags.append(t)
                problem.tags = problem.topics = tags

            difficulty_i, difficulty_c = find_section('#+\s*Difficulty.*', lines)
            if difficulty_i:
                problem.difficulty = float(difficulty_c[difficulty_i[0]][0])

            sample_input_i, sample_input_c = find_section('#+\s*Sample input(.*)', lines)
            sample_output_i, sample_output_c = find_section('#+\s*Sample (ou|Ou|out|Out)put(.*)', lines)
            explanation_i, explanation_c = find_section('#+\s*Explanation(.*)', lines)
            if sample_input_i:
                for i in range(len(sample_input_i)):
                    testcase = TestCase()
                    testcase.input = join_lines(sample_input_c[sample_input_i[i]]).strip('`').strip()
                    testcase.output = join_lines(sample_output_c[sample_output_i[i]]).strip('`').strip()
                    if len(explanation_i) > i:
                        testcase.explanation = join_lines(explanation_c[explanation_i[i]]).strip('`').strip()

                    problem.testcases.append(testcase)

        solution_file = os.path.join(problem_folder, f"{problem_code}.py")
        if os.path.isfile(solution_file):
            with open(solution_file) as f:
                problem.solution = f.read()

        if load_testcase:
            file_names = ["testcases_manual_stock.txt", "testcases_manual.txt", "testcases_stock.txt", "testcases.txt"]
            for file_name in file_names:
                testcase_file = os.path.abspath(os.path.join(problem_folder, file_name))

                if not os.path.exists(testcase_file):
                    CLog.warn(f'`{testcase_file}` file not existed, skipping...')
                else:
                    tests = read_testcases_from_file(testcase_file)
                    print(f"reading testcases in file {testcase_file}: {len(tests)}")
                    for t in tests:
                        input, output = t['input'], t['output']
                        print(f"Input: {input}, output: {output}")
                        if input not in [test.input for test in problem.testcases]:
                            problem.testcases.append(TestCase(input=input, output=output))

        if not problem.testcases_sample and problem.testcases:
            problem.testcases_sample = problem.testcases[:1]

        return problem

    @staticmethod
    def make_problem_code(name):
        name = unidecode(name)
        name = name\
            .replace(" ", "_") \
            .replace(":", "_") \
            .replace("!", "") \
            .replace("%", "") \
            .replace('-', '_') \
            .replace(' ', '_') \
            .replace('.', '_') \
            .replace("'", '') \
            .replace("’", '') \
            .replace('"', '_') \
            .replace('__', '_') \
            .lower() \
            .strip(" _")
        if '0' <= name[0] <= '9':
            name = "p" + name
        return name

    @staticmethod
    def save(problem : Problem, base_folder=".", problem_code=None, overwrite=False):
        """
        :param problem:
        :param problem_code: name of problem folder
        :param base_folder: the parent folder that will contain the problem folder
        :return:
        """
        if not problem_code:
            problem_code = problem.name

        problem_code = DsaProblem.make_problem_code(problem_code)

        problem.code = problem_code

        problem_folder = os.path.join(base_folder, problem_code)

        if os.path.exists(problem_folder):
            if not overwrite:
                CLog.error('Problem folder existed! Delete the folder or use `overwrite` instead.')
                return
            else:
                CLog.warn('Problem folder existed! Content will be overwritten.')

        if not os.path.exists(problem_folder):
            os.makedirs(problem_folder)

        # if not os.path.exists(problem_folder + "/testcases"):
        #     os.makedirs(problem_folder + "/testcases")

        if not problem.name:
            problem.name = (' '.join(problem_code.split('_'))).title()

        template_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'templates')

        if problem.testcases:
            with open(os.path.join(template_path, 'testcases.txt.j2')) as file_:
                template = Template(file_.read())
                content = template.render(testcases=problem.testcases)
                f = open(problem_folder + "/testcases.txt", 'w')
                f.write(content)
                f.close()

        if problem.testcases_sample:
            with open(os.path.join(template_path, 'testcases.txt.j2')) as file_:
                template = Template(file_.read())
                content = template.render(testcases=problem.testcases_sample)
                f = open(problem_folder + "/testcases_manual.txt", 'w')
                f.write(content)
                f.close()

        if problem.stock_testcases:
            with open(os.path.join(template_path, 'testcases.txt.j2')) as file_:
                template = Template(file_.read())
                content = template.render(testcases=problem.stock_testcases)
                f = open(problem_folder + "/testcases_stock.txt", 'w')
                f.write(content)
                f.close()

        if problem.stock_testcases_sample:
            with open(os.path.join(template_path, 'testcases.txt.j2')) as file_:
                template = Template(file_.read())
                content = template.render(testcases=problem.stock_testcases_sample)
                f = open(problem_folder + "/testcases_manual_stock.txt", 'w')
                f.write(content)
                f.close()

        if not problem.testcases_sample:
            problem.testcases_sample = problem.stock_testcases_sample

        with open(os.path.join(template_path, 'statement.md.j2')) as file_:
            template = Template(file_.read())
            statement = template.render(problem=problem)
            f = codecs.open(problem_folder + ("/%s.md" % problem_code), "w", "utf-8")
            f.write(statement)
            f.close()
            f = codecs.open(problem_folder + ("/%s.vi.md" % problem_code), "w", "utf-8")
            f.write(statement)
            f.close()

        with open(os.path.join(template_path, 'editorial.md.j2')) as file_:
            template = Template(file_.read())
            statement = template.render(problem=problem)
            f = codecs.open(problem_folder + ("/%s_editorial.md" % problem_code), "w", "utf-8")
            f.write(statement)
            f.close()

        with open(os.path.join(template_path, 'solution.py.j2')) as file_:
            template = Template(file_.read())
            content = template.render(problem_code=problem_code,
                                      solution=problem.solution if problem.solution else "pass")
            f = open(problem_folder + ("/%s.py" % problem_code), 'w')
            f.write(content)
            f.close()

        with open(os.path.join(template_path, 'generator.py.j2')) as file_:
            template = Template(file_.read())
            content = template.render(problem_code=problem_code)
            f = open(problem_folder + ("/%s_generator.py" % problem_code), 'w')
            f.write(content)
            f.close()

        if problem.solutions:
            for solution in problem.solutions:
                f = open(problem_folder + ("/solution.%s" % solution["lang"]), 'w')
                f.write(solution["code"])
                f.close()

        problem_folder = os.path.abspath(problem_folder)

        CLog.important(f'Problem created at `{problem_folder}`')


if __name__ == "__main__":
    # load_problem('/home/thuc/teko/online-judge/dsa-problems/number_theory/num001_sumab')
    # load_problem('/home/thuc/teko/online-judge/dsa-problems/unsorted/minhhhh/m010_odd_to_even')
    # problem = load_problem('/home/thuc/teko/online-judge/ptoolbox/problems/array001_counting_sort3')
    dsa_problem = DsaProblem.load('/home/thuc/teko/online-judge/dsa-problems/number_theory/num001_sumab')
    print(dsa_problem.prints())




import random
import string


class CRandom:
    @staticmethod
    def string(len, seed=string.ascii_uppercase + string.ascii_lowercase):
        return ''.join(random.choices(seed, k=len))

    @staticmethod
    def list_int(len, vmin, vmax, distinct = False):
        if not distinct:
            res = []
            for i in range(0, len):
                res.append(CRandom.int(vmin, vmax))
            return res
        else:
            return random.sample(range(vmin, vmax+1), len)

    @staticmethod
    def int(min, max):
        return random.randint(min, max)


if __name__ == '__main__':
    print(CRandom.string(10))
    print(*CRandom.list_int(10, -10, 10, distinct=True))
    print(CRandom.int(1,10))



import copy
import json
import pprint
import shutil
import time
from zipfile import ZipFile
import os

import configparser
import requests
from ptoolbox.dsa import dsa_problem_file
from ptoolbox.dsa.dsa_problem import DsaProblem
from ptoolbox.helpers.clog import CLog
from ptoolbox.models.general_models import Problem


def chomp(s):
    # if s.endswith("\r\n"): return s[:-2]
    # if s.endswith("\n"): return s[:-1]
    return s.strip()


def save_testcase_file(testcase_folder, tc_number, tc_input, tc_output):
    inp_folder = os.path.join(testcase_folder, "input")
    out_folder = os.path.join(testcase_folder, "output")

    if not os.path.exists(inp_folder):
        os.makedirs(inp_folder)

    CLog.echo(f"Writing testcase #{tc_number}")
    CLog.echo("   input...")

    f = open(os.path.join(inp_folder, "input%02d.in" % tc_number), "w")
    f.write(tc_input)
    f.close()

    if not os.path.exists(out_folder):
        os.makedirs(out_folder)
    CLog.echo("   output...")
    f = open(os.path.join(out_folder, "output%02d.out" % tc_number), "w")
    f.write(tc_output)
    f.close()


def prepare_testcases_from_file(tc_file, testcase_output_folder, start_number=0):
    testcases = dsa_problem_file.read_testcases_from_file(tc_file)

    for i in range(len(testcases)):
        save_testcase_file(testcase_output_folder, start_number+i, testcases[i]['input'], testcases[i]['output'])

    return start_number + len(testcases)


def prepare_testcases(problem_folder, keep_zip_file_only=True):
    zipfilename = 'testcases_hackerrank.zip'
    testcase_output_folder = os.path.abspath(os.path.join(problem_folder, "testcases"))

    if os.path.exists(os.path.join(testcase_output_folder, zipfilename)):
        CLog.warn('`testcases.zip` file existed, will be overwritten.')

    file_names = ["testcases_manual_stock.txt", "testcases_manual.txt", "testcases_stock.txt", "testcases.txt"]

    count = 0

    for file_name in file_names:
        testcase_file = os.path.abspath(os.path.join(problem_folder, file_name))

        if not os.path.exists(testcase_file):
            CLog.warn(f'`{testcase_file}` file not existed, skipping...')
        else:
            count = prepare_testcases_from_file(testcase_file, testcase_output_folder, count)

    print("DONE")

    if count:
        zip_test_cases(problem_folder, count, zipfilename)

        if keep_zip_file_only:
            CLog.echo(f"Deleting intermediate files...")
            shutil.rmtree(testcase_output_folder)
            print("DONE")
    else:
        CLog.error(f'No testcases found in {os.path.abspath(problem_folder)}')

    return count


def zip_test_cases(problem_folder, count, zipfilename):
    problem_folder = os.path.abspath(os.path.join(problem_folder, "testcases"))
    zipfile = os.path.join(problem_folder, '..', zipfilename)
    print(f"creating `{os.path.abspath(zipfile)}`")

    os.chdir(problem_folder)
    with ZipFile(zipfile, 'w') as myzip:
        for i in range(count):
            myzip.write("input/input%02d.in" % i)
            myzip.write("output/output%02d.out" % i)

    print("DONE")


class HackerRank:
    def __init__(self):
        self.username = None
        self.csrf_token = None
        self.s = requests.session()
        self._headers = {
            'origin': 'https://www.hackerrank.com',
            'referer': 'https://www.hackerrank.com/dashboard',
            'user-agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) '
                          'Chrome/70.0.3538.77 Safari/537.36',
        }

    @staticmethod
    def read_credential(credential_file):
        config = configparser.ConfigParser()
        config.read(credential_file)
        if not config.has_section('HACKERRANK'):
            CLog.error(f'Section `HACKERRANK` should exist in {credential_file} file')
            return None, None
        if not config.has_option('HACKERRANK', 'username') or not config.has_option('HACKERRANK', 'password'):
            CLog.error(f'Username and/or password are missing in {credential_file} file')
            return None, None

        username = config.get('HACKERRANK', 'username')
        password = config.get('HACKERRANK', 'password')

        return username, password

    @staticmethod
    def get_csrf(html):
        s = 'name="csrf-token"'
        i = html.find(s)
        if i < 0:
            raise ValueError('Cannot get csrf token')

        i = html[:i].rfind('<meta')
        i = html.find('content="', i + 1)
        i += len('content="')
        j = html.find('"', i + 1)
        csrf = html[i: j]
        if not csrf:
            raise ValueError('Cannot get csrf token')
        return csrf

    def login(self, username, password):
        url = 'https://www.hackerrank.com/rest/auth/login'

        headers = copy.deepcopy(self._headers)
        # headers['content-type'] = 'application/json' # automatically overwrite when set json = python dict

        r = self.s.get('https://www.hackerrank.com/auth/login', headers=headers)

        data = {
            "login": username,
            "password": password,
            "remember_me": False,
            "fallback": False
        }
        self.csrf_token = HackerRank.get_csrf(r.text)
        # print("csrf:", self.csrf_token)
        headers['x-csrf-token'] = self.csrf_token
        r = self.s.post(url, json=data, headers=headers)

        print(r.status_code)
        # print(r.headers)
        # print(r.text)
        raw = r.json()
        print(raw)

        if raw['status']:
            # print('Login succeeded, csrf token: {}'.format(raw['csrf_token']))
            CLog.important('Login succeeded')
            self.username = username
            self.csrf_token = raw['csrf_token']
        else:
            raise ValueError("Login failed with user: {}".format(username))

        # res = r.json()origin
        # print(res)

        return True

    def get_raw_track(self, track_url, offset=0, limit=10, save_to_file=None):
        # hackkerrank chi co lay 50 cai 1
        size = min(limit, 50)
        i = 0
        print(size, limit)

        raw_track = None
        while i < limit:
            url = track_url + '?offset={}&limit={}'.format(offset, min(limit - i, size))
            headers = copy.deepcopy(self._headers)
            print("get url: " + url)
            r = self.s.get(url, headers=headers)

            # print(r.text)

            raw = r.json()
            if len(raw['models']) < 1:
                break

            if i == 0:
                raw_track = raw
            else:
                raw_track['models'].extend(raw['models'])

            offset += 50
            i += min(limit - i, size)

        if save_to_file:
            with open(save_to_file, 'w') as outfile:
                json.dump(raw_track, outfile)

        return raw_track

    @staticmethod
    def load_raw_track_from_file(filename):
        with open(filename) as f:
            return json.load(f)

    @staticmethod
    def parse_track(raw_track):
        hkr_track = HackerRankTrack()
        hkr_track.total = raw_track['total']
        if 'current_track' in raw_track:
            d = raw_track['current_track']
            hkr_track.src_id = d['id']
            hkr_track.name = d['name']
            hkr_track.slug = d['slug']
            hkr_track.descriptions = d['descriptions']

        hkr_track.problems = raw_track['models']
        hkr_track.problems = []
        for raw_prob in raw_track['models']:
            problem = Problem()
            problem.src_id = raw_prob['id']
            problem.name = raw_prob['name']
            problem.slug = raw_prob['slug']
            problem.category = raw_prob['category']
            problem.type = raw_prob['kind']
            problem.preview = raw_prob['preview']
            problem.tags = raw_prob['tag_names']
            problem.topics = raw_prob['topics']
            problem.hints = raw_prob['hints']
            problem.difficulty = raw_prob['difficulty']
            problem.difficulty_level = raw_prob['difficulty_name']
            problem.contest_slug = raw_prob['contest_slug']
            problem.total_count = raw_prob['total_count']
            problem.solved_count = raw_prob['solved_count']
            problem.success_ratio = raw_prob['success_ratio']
            problem.track = HackerRankTrack('', raw_prob['track']['id'], raw_prob['track']['name'],
                                            raw_prob['track']['slug'])

            hkr_track.problems.append(problem)
        return hkr_track

    def get_problem(self, problem_slug):
        url = 'https://www.hackerrank.com/rest/contests/master/challenges/{}/'.format(problem_slug)
        headers = copy.deepcopy(self._headers)
        r = self.s.get(url, headers=headers)

        print(r.text)

        raw = r.json()
        # problem = self.parse_problem(raw)
        return raw

    def submit(self, problem_slug, language, code, compile_test=True, custominput=None):
        contest_slug = 'master'  # TODO: change this
        if compile_test:
            url = 'https://www.hackerrank.com/rest/contests/{}/challenges/{}/compile_tests'.format(contest_slug,
                                                                                                   problem_slug)
        else:
            url = 'https://www.hackerrank.com/rest/contests/{}/challenges/{}/submissions'.format(contest_slug,
                                                                                                 problem_slug)

        headers = copy.deepcopy(self._headers)
        headers['x-csrf-token'] = self.csrf_token
        # TODO: https://www.hackerrank.com/contests/{}/challenges/{}.format(contest_slug, problem_slug)
        headers['referer'] = 'https://www.hackerrank.com/challenges/{}/problem'.format(problem_slug)

        data = {
            "code": code,
            "language": language,
            "playlist_slug": '',
        }

        if compile_test:
            if custominput:
                data["customtestcase"] = True
                data['custominput'] = custominput
            else:
                data["customtestcase"] = False
        else:  # real submission
            data["contest_slug"] = 'master'

        print('sending submission:', url)
        print(headers)

        r = self.s.post(url, json=data, headers=headers)

        print(r.text)

        raw = r.json()
        if not raw['status']:
            return None
        # problem = self.parse_problem(raw)
        return raw['model']['id']

    def check_submission_result(self, problem_slug, submission_id, compile_test=True, interval=1):
        contest_slug = 'master'  # TODO: change this

        count = 0
        while count < 10:
            time.sleep(interval)
            count += 1
            if compile_test:
                url = 'https://www.hackerrank.com/rest/contests/{}/challenges/{}/compile_tests/{}' \
                    .format(contest_slug, problem_slug, submission_id)
            else:
                url = 'https://www.hackerrank.com/rest/contests/{}/challenges/{}/submissions/{}' \
                    .format(contest_slug, problem_slug, submission_id)

            headers = copy.deepcopy(self._headers)
            print('Checking %d... %s' % (count, url))
            r = self.s.get(url, headers=headers)

            print(r.text)

            raw = r.json()
            # see hackerrank-sample-compile-result-*.json for examples
            if compile_test and raw['model']['status'] == 1:  # compile_test mode
                return raw

            if not compile_test and raw['model']['status_code'] == 1:  # judge mode
                return raw

        return None

    def create_problem(self, problem):
        url = 'https://www.hackerrank.com/rest/administration/challenges'

        headers = copy.deepcopy(self._headers)
        headers['x-csrf-token'] = self.csrf_token
        headers['referer'] = 'https://www.hackerrank.com/administration/challenges/create'

        data = {
            "name": problem.name,
            "preview": problem.preview,
            "problem_statement_fields": {
                'problem_statement': problem.statement,
                'input_format': problem.input_format,
                'constraints': problem.constraints,
                'output_format': problem.output_format,
            },
            # "tags": ["basic"],
            "tags": problem.tags,
        }

        print('creating problem:')
        print(data)

        r = self.s.post(url, json=data, headers=headers)
        print(r.status_code)
        print(r.headers)
        print(r.text)

        raw = r.json()
        if not raw['status']:
            print(raw['errors'])
            raise ValueError("Cannot create challenge")
        # problem = self.parse_problem(raw)
        problem.src_id = raw['model']['id']
        problem.slug = raw['model']['slug']
        CLog.important(f'Hackerrank problem created: {problem.src_id} slug: {problem.slug}')
        return raw['model']

    @staticmethod
    def parse_problem(raw_problem):
        r = raw_problem['model']
        problem = Problem()
        problem.src_id = r['id']
        problem.name = r['name']
        problem.slug = r['slug']
        problem.preview = r['preview']
        problem.type = r['kind']
        problem.languages = r['languages']
        problem.public_test_cases = r['public_test_cases']
        problem.public_solutions = r['public_solutions']
        problem.difficulty = r['difficulty']
        return problem

    def update_problem(self, problem_id, problem):
        url = 'https://www.hackerrank.com/rest/administration/challenges/{}'.format(problem_id)

        headers = copy.deepcopy(self._headers)
        headers['x-csrf-token'] = self.csrf_token
        headers['referer'] = 'https://www.hackerrank.com/administration/challenges/create'

        data = {
            "name": problem.name,
            "id": problem_id,
            'difficulty': problem.difficulty,
            "preview": problem.preview,
            "problem_statement_fields": {
                'problem_statement': problem.statement,
                'input_format': problem.input_format,
                'constraints': problem.constraints,
                'output_format': problem.output_format,
            },
            'slug': problem.slug,
            "tags": problem.tags,
            'public_solutions': problem.public_solutions,
            'public_test_cases': problem.public_test_cases,
            'translation_language': 'English'
        }

        print('updating problem:')
        print(data)

        r = self.s.put(url, json=data, headers=headers)

        print(r.text)

        raw = r.json()
        if not raw['status']:
            print(raw['errors'])
            raise ValueError("Cannot create challenge")
        # problem = self.parse_problem(raw)

        if problem.languages:
            url = 'https://www.hackerrank.com/rest/administration/challenges/{}/allowed_languages'\
                .format(problem.src_id)
            data = []
            for lang in problem.languages:
                data.append(('languages[]', lang))
            print('updating languages:')
            print(data)
            r1 = self.s.put(url, data=data, headers=headers)
            if r1.json()[0] != 'ok':
                print(r1.text)
                raise ValueError('Cannot set allowed languages')
        return raw['model']

    def upload_testcases(self, problem_id, zip_file_path):
        """

        :param problem_id:
        :param zip_file_path:
        :return: list of created hackerrank testcases
        [
           {
              "id":1161797,
              "challenge_id":113357,
              "input":"input00.txt",
              "output":"output00.txt",
              "score":10.0,
              "sample":false,
              "tag":null,
              "input_size":10,
              "output_size":2,
              "additional":null,
              "h_input_size":"10 Bytes",
              "h_output_size":"2 Bytes",
              "explanation":null
           },
           {
              "id":1161798,
              "challenge_id":113357,
              "input":"input01.txt",
              "output":"output01.txt",
              "score":10.0,
              "sample":false,
              "tag":null,
              "input_size":9,
              "output_size":1,
              "additional":null,
              "h_input_size":"9 Bytes",
              "h_output_size":"1 Byte",
              "explanation":null
           }
        ]
        """
        url = 'https://www.hackerrank.com/rest/administration/challenges/{}/test_cases/upload_zip'.format(problem_id)

        headers = copy.deepcopy(self._headers)
        headers['x-csrf-token'] = self.csrf_token
        headers['referer'] = 'https://www.hackerrank.com/administration/challenges/edit/{}/testcases'.format(problem_id)

        files = {'zip_file': ('testcases.zip', open(zip_file_path, 'rb'), 'application/zip')}
        print('uploading file: ', files)
        r = self.s.post(url, files=files, headers=headers)

        print(r.text)
        raw = r.json()
        if not raw['status']:
            print(raw['errors'])
            raise ValueError("Error uploading test cases")
        return raw['models']

    def update_testcases_score(self, problem_id, testcase_ids, score=100, sample_ids=None):
        """

        :param problem_id:
        :param testcase_ids:
        :param score:
        :param sample_ids: list of testcase ids to mark as sample
        :return:
        """
        url = 'https://www.hackerrank.com/rest/administration/challenges/{}/test_cases/{}'\
            .format(problem_id, ','.join([str(t) for t in testcase_ids]))
        print('Updating testcase score... {}'.format(url))

        changes = {
            'auto_generate_sample_cases': True,
            'test_cases': dict()
        }

        for t in testcase_ids:
            is_sample = True if t in sample_ids else False
            changes['test_cases'][t] = {
                'sample': is_sample,
                'additional': False,
                'score': 0 if is_sample else score
            }

        data = {
            'changes': json.dumps(changes)
        }

        print(data)

        headers = copy.deepcopy(self._headers)
        headers['x-csrf-token'] = self.csrf_token
        headers['referer'] = 'https://www.hackerrank.com/administration/challenges/edit/{}/testcases'.format(problem_id)

        r = self.s.put(url, data=data, headers=headers)
        print(r.text)

        raw = r.json()
        if not raw['status']:
            print(raw['errors'])
            raise ValueError("Error updating testcases")
        return raw['models']

    def upload_testcases_and_set_score(self, problem_id, zip_file_path, score=100, sample_test_count=1):
        testcases = self.upload_testcases(problem_id, zip_file_path)
        testcase_ids = [t['id'] for t in testcases]
        sample_ids = [testcase_ids[i] for i in range(min(len(testcase_ids), sample_test_count))]
        print('testcase_ids:', testcase_ids)
        print('sample_ids:', testcase_ids)
        return self.update_testcases_score(problem_id, testcase_ids, score, sample_ids)


class HackerRankTrack:
    def __init__(self, url=None, track_id=None, name=None, slug=None, descriptions=None):
        self.src_id = track_id
        self.name = name
        self.slug = slug
        self.descriptions = descriptions
        self.url = url
        self.problems = []
        self.problem_count = 0
        self.total = 0

    def __str__(self):
        self.problem_count = len(self.problems)
        tmp = copy.deepcopy(self)
        tmp.problems = None
        return pprint.pformat(vars(tmp), indent=2)

    @staticmethod
    def track_list():
        tracks = [
            HackerRankTrack('https://www.hackerrank.com/rest/contests/master/tracks/algorithms/challenges'),
            HackerRankTrack('https://www.hackerrank.com/rest/contests/master/tracks/data-structures/challenges')
        ]
        return tracks


if __name__ == "__main__":
    # prepare_testcases("../problems/prob1", True)
    hr = HackerRank()
    hr.login('thucngch', '15041985')

    problem_dir = '/home/thuc/teko/online-judge/ptoolbox/problems/array001_counting_sort3'

    problem1 = DsaProblem.load(problem_dir)

    problem1 = hr.update_problem('113357', problem1)
    # problem1 = hr.create_problem(problem1)
    # print('HackerrankID:', problem1.src_id)
    # 113357

    prepare_testcases(problem_dir)
    testcase_zip = os.path.join(problem_dir, 'testcases_hackerrank.zip')
    created_testcases = hr.upload_testcases_and_set_score('113357', testcase_zip, 50, 4)




import click


class CLog:
    @staticmethod
    def _log(msg, level=None, fg=None, bg=None, bold=None, dim=None, underline=None, blink=None, reverse=None):
        if level:
            msg = f'[{level}]: {msg}'
        click.echo(click.style(msg, fg=fg, bg=bg, bold=bold, dim=dim,
                               underline=underline, blink=blink, reverse=reverse))

    @staticmethod
    def trivial(msg, **kwargs):
        if 'dim' not in kwargs:
            kwargs['dim'] = True
        CLog._log(msg, **kwargs)

    @staticmethod
    def echo(msg, **kwargs):
        CLog._log(msg, **kwargs)

    @staticmethod
    def info(msg, **kwargs):
        if 'fg' not in kwargs:
            kwargs['fg'] = 'bright_blue'
        CLog._log(msg, 'INFO', **kwargs)

    @staticmethod
    def important(msg, **kwargs):
        if 'fg' not in kwargs:
            kwargs['fg'] = 'green'
        if 'bold' not in kwargs:
            kwargs['bold'] = True
        CLog._log(msg, 'IMPORTANT', **kwargs)

    @staticmethod
    def warn(msg, **kwargs):
        if 'fg' not in kwargs:
            kwargs['fg'] = 'yellow'
        CLog._log(msg, 'WARN', **kwargs)

    @staticmethod
    def error(msg, **kwargs):
        if 'fg' not in kwargs:
            kwargs['fg'] = 'bright_red'
        CLog._log(msg, 'ERROR', **kwargs)

    @staticmethod
    def fatal(msg, **kwargs):
        if 'fg' not in kwargs:
            kwargs['fg'] = 'bright_red'
        if 'bold' not in kwargs:
            kwargs['bold'] = True
        if 'reverse' not in kwargs:
            kwargs['reverse'] = True
        CLog._log(msg, 'FATAL', **kwargs)


if __name__ == '__main__':
    CLog._log('log')
    CLog.trivial('trivial')
    CLog.echo('echo')
    CLog.info('info')
    CLog.info('info', bold=True)
    CLog.important('important')
    CLog.warn('warn')
    CLog.error('error')
    CLog.fatal('fatal')

